#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common

def extract_sentences(phrase_table_file, biltrans_out_file):
    lineno = 0
    total_valid = 0
    total_errors = 0

    not_ambiguous = []
    with open(phrase_table_file) as phrase_table, open(biltrans_out_file) as biltrans_out:
        while True:
            try:
                lineno = lineno + 1
                pt_line = phrase_table.readline().strip()
                bt_line = biltrans_out.readline().strip()

                if not bt_line.strip() and not pt_line.strip():
                    break
                elif not bt_line.strip() or not pt_line.strip():
                    continue

                row = pt_line.split('|||')
                bt = common.tokenise_biltrans_line(bt_line.strip())
                sl = common.tokenise_tagger_line(row[1].strip())
                tl = common.tokenise_tagger_line(row[0].strip())

                if not common.ambiguous(bt):
                    not_ambiguous.append(str(lineno))
                    if len(not_ambiguous) >= 10:
                        print("not ambiguous:", ' '.join(
                            not_ambiguous), file=sys.stderr)
                        not_ambiguous = []

                    continue

                if len(sl) < 2 and len(tl) < 2:
                    continue

                # Check that the number of words in the lexical transfer, and in the phrasetable matches up
                if len(sl) != len(bt):
                    print("Error in line", lineno,
                          ": len(sl) != len(bt)", file=sys.stderr)
                    continue

                # cheking if the alignments are empty
                if not row[2].strip():
                    print("In line", lineno,
                          ", alignments are empty", file=sys.stderr)
                    continue

                # Resumption<n> of<pr> the<def><def> session<n>
                # Resumption<n><sg>/Reanudación<n><f><sg> of<pr>/de<pr> the<det><def><sp>/el<det><def><GD><ND> session<n><sg>/sesión<n><f><sg>
                # Reanudación<n> de<pr> el<det><def> periodo<n> de<pr> sesión<n>
                # 0-0 1-1 2-2 5-3

                print(lineno, '\t' + row[1])
                print(lineno, '\t' + bt_line)
                print(lineno, '\t' + row[0])
                print(lineno, '\t' + row[2])
                print('-'*(len(bt_line)+5))
                total_valid += 1
            except Exception as e:
                print("Error in line", lineno, ": ", e, file=sys.stderr)
                total_errors += 1
                continue

    print('total:', lineno, file=sys.stderr)
    print('valid:', total_valid,
          '(' + str((total_valid/lineno)*100) + '%)', file=sys.stderr)
    print('errors:', total_errors,
          '(' + str((total_errors/lineno)*100) + '%)', file=sys.stderr)


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: extact-sentences.py <phrasetable> <biltrans>', file=sys.stderr)
        exit(1)

    extract_sentences(sys.argv[1], sys.argv[2])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

from collections import defaultdict
import common
import math
import re
import sys


def safe_float(s):
    if not s:
        return 0.0, False
    try:
        f = float(s)
        if math.isnan(f):
            return 0.0, False
        return f, True
    except:
        return 0.0, False


def strip_tags(s, side, space=False):
    idx = 0 if side == 'sl' else 1
    ret = s.split('/')[idx]
    if not space:
        ret = ret.replace(' ', '~')
    if ret.count('><') > 0:
        ret = ret.split('><')[0] + '>'
    return ret


class BiltransCounter:
    lu_sep = re.compile(r'\$[^\^]*\^')
    tokenizer = 'regex'  # or 'biltrans'
    line_ids = False
    count_ngrams = False
    max_ngrams = 3
    biltrans_wrap_lus = False

    def __init__(self):
        self.reading = False

        self.am_file = None
        self.am_line = None
        self.am_row = None
        self.am_id = None
        self.am_linenum = 0
        self.dm_file = None
        self.dm_line = None
        self.dm_row = None
        self.dm_id = None
        self.dm_linenum = 0

        self.clear_ngrams()

    def __del__(self):
        if self.am_file:
            self.am_file.close()
        if self.dm_file:
            self.dm_file.close()

    def next_am_line(self):
        self.am_line = self.am_file.readline()
        self.am_linenum += 1
        if not self.am_line:
            self.am_id, self.am_row = None, []
            self.reading = False
            return
        ls = self.am_line.split('\t')
        if self.line_ids:
            self.am_id = int(ls[0].strip())
        if self.tokenizer == 'regex':
            self.am_row = self.lu_sep.split(ls[1].strip()[1:-1])
        elif self.tokenizer == 'biltrans':
            self.am_row = common.tokenize_biltrans_line(self.am_line)

    def next_dm_line(self):
        self.dm_linenum += 1
        self.dm_line = self.dm_file.readline()
        if not self.dm_line:
            self.dm_id, self.dm_row = None, []
            self.reading = False
            return
        ls = self.dm_line.split('\t')
        if self.line_ids:
            self.dm_id = int(self.dm_line.split('.[][')[1].split()[0])
        if self.tokenizer == 'regex':
            self.dm_row = self.lu_sep.split(ls[1].strip()[1:-1])
        elif self.tokenizer == 'biltrans':
            self.dm_row = common.tokenize_biltrans_line(self.dm_line)

    def clear_ngrams(self):
        self.ngrams = defaultdict(
            lambda: defaultdict(lambda: defaultdict(lambda: 0.0)))

    def check_rows(self):
        if len(self.am_row) != len(self.dm_row):
            print(
                'Mismatch in number of LUs between analysis and training', file=sys.stderr)
            print('\t' + self.am_line, file=sys.stderr)
            print('\t' + self.dm_line, file=sys.stderr)
            print('...skipping', file=sys.stderr)
            return False
        return True

    def read_files_multi_dm(self, am_fname, dm_fname):
        self.next_dm_line()
        while self.reading:
            self.next_am_line()
            while self.am_id == self.dm_id and self.reading:
                frac_count = 0
                if self.dm_line.count('\t') > 1:
                    frac_count, _ = safe_float(self.dm_line.split('\t')[2])
                if self.check_rows():
                    self.process_row(frac_count)
                self.next_dm_line()
            if self.am_linenum % 1000 == 0:
                print('=> %d SL and %d TL lines read' %
                      (self.am_linenum, self.dm_linenum), file=sys.stderr)

    def read_files(self, am_fname, dm_fname):
        self.am_file = open(am_fname)
        self.dm_file = open(dm_fname)
        self.reading = True
        if self.line_ids:
            self.read_files_multi_dm(am_fname, dm_fname)
            return
        while self.reading:
            self.next_am_line()
            self.next_dm_line()
            if self.reading and self.check_rows():
                self.process_row()
            if self.am_linenum % 1000 == 0:
                print('=> %d lines read' % self.am_linenum, file=sys.stderr)

    def process_row(self, frac_count=0):
        if self.tokenizer == 'regex':
            cur_sl_row = [strip_tags(s, 'sl', space=True) for s in self.am_row]
            for i in range(len(self.am_row)):
                if self.am_row[i].count('/') > 1:
                    sl = strip_tags(self.am_row[i], 'sl')
                    tl = strip_tags(self.dm_row[i], 'tl')
                    self.process_lu_internal(sl, tl, i, cur_sl_row, frac_count)
        elif self.tokenizer == 'biltrans':
            cur_sl_row = [x['sl'] for x in self.am_row]
            for i in range(len(self.am_row)):
                if len(self.am_row[i]['tls']) > 1:
                    sl = self.am_row[i]['sl']
                    tl = self.dm_row[i]['tls'][0]
                    if self.biltrans_wrap_lus:
                        sl = common.wrap(sl)
                        tl = common.wrap(tl)
                    self.process_lu_internal(sl, tl, i, cur_sl_row, frac_count)

    def process_lu_internal(self, sl, tl, idx, cur_sl_row, frac_count=0):
        if self.count_ngrams:
            for j in range(1, self.max_ngrams):
                pregram = ' '.join(map(common.wrap, cur_sl_row[idx-j:idx+1]))
                postgram = ' '.join(map(common.wrap, cur_sl_row[idx:idx+j+1]))
                roundgram = ' '.join(
                    map(common.wrap, cur_sl_row[idx-j:idx+j+1]))
                self.ngrams[sl][pregram][tl] += frac_count
                self.ngrams[sl][postgram][tl] += frac_count
                self.ngrams[sl][roundgram][tl] += frac_count
        self.process_lu(sl, tl, idx, cur_sl_row, frac_count)

    def process_lu(self, sl, tl, idx, cur_sl_row, frac_count=0):
        pass


def features_and_outline(ngrams, sl, tl, sl_tl, features, indexes,
                         frac_count=None):
    if not ngrams[sl]:
        return
    meevents = []
    for ni in ngrams[sl]:
        if ni not in features:
            feature_counter = len(features) + 1
            features[ni] = feature_counter
        meevents.append(features[ni])
    if sl not in sl_tl or len(sl_tl[sl]) < 2:
        return
    outline = str(indexes[(sl, tl)])
    if frac_count != None:
        outline += ' $ ' + str(int(frac_count * 10000)) + ' '
    outline += ' # '
    for j in range(len(sl_tl[sl])):
        for feature in meevents:
            outline += '%s:%s ' % (feature, j)
        outline += ' # '
    print('%s\t%s\t%s' % (sl, len(sl_tl[sl]), outline))


def read_frequencies(fname):
    with open(fname) as fin:
        sl_tl = {}
        sl_tl_defaults = {}
        indexes = {}
        trad_counter = defaultdict(lambda: 0)
        for line_ in fin.readlines():
            line = line_.strip()
            if not line:
                continue
            row = common.tokenize_tagger_line(line)
            sl = row[0]
            tl = row[1]
            fr = float(line.split(' ')[0])
            indexes[(sl, tl)] = trad_counter[sl]
            trad_counter[sl] += 1
            if '@' in line:
                sl_tl_defaults[sl] = tl
                if fr == 0.0:
                    print(
                        '!!! Prolly something went wrong here, the default has freq of 0.0', file=sys.stderr)
                else:
                    print('    %s => %s = %.10f' %
                          (sl, tl, fr), file=sys.stderr)
            else:
                sl_tl[sl] = tl
        return sl_tl, sl_tl_defaults, indexes



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import biltrans_count_common as BCC

# Input:
#        a) Frequency lexicon

# 0.9983989491 kemennadur<n> précepte<n> @
# 0.0016010509 kemennadur<n> directive<n>
# 2.9999998997 a-led<adv> horizontal<adv> @
# 0.0000001004 a-led<adv> à~plat<adv>
# 20.4545497200 pazenn<n> étape<n> @
# 2.5454502800 pazenn<n> marche<n>
#

#        b) Biltrans output

# 56011   ^un<det><ind><sp>/un<det><ind><GD><ND>$ ^digarez<n><m><sg>/excuse<n><f><sg>/occasion<n><f><sg>$ ^da<pr>/à<pr>$ ^distreiñ<vblex><inf>/revenir<vblex><inf>$ ^war<pr>/sur<pr>$ ^e<det><pos><m><sp>/son<det><pos><GD><ND>$ ^doare<n><m><sg>/manière<n><f><sg>$ ^ober<vblex><inf>/faire<vblex><inf>$ ^.<sent>/.<sent>$


#        c) Disambiguated biltrans output

# .[][56011 0].[] ^un<det><ind><sp>/un<det><ind><GD><ND>$ ^digarez<n><m><sg>/excuse<n><f><sg>$ ^da<pr>/à<pr>$ ^distreiñ<vblex><inf>/revenir<vblex><inf>$ ^war<pr>/sur<pr>$ ^e<det><pos><m><sp>/son<det><pos><GD><ND>$ ^doare<n><m><sg>/manière<n><f><sg>$ ^ober<vblex><inf>/faire<vblex><inf>$ ^.<sent>/.<sent>$^.<sent>/.<sent>$ 0.9917274061    |@|
# .[][56011 1].[] ^un<det><ind><sp>/un<det><ind><GD><ND>$ ^digarez<n><m><sg>/occasion<n><f><sg>$ ^da<pr>/à<pr>$ ^distreiñ<vblex><inf>/revenir<vblex><inf>$ ^war<pr>/sur<pr>$ ^e<det><pos><m><sp>/son<det><pos><GD><ND>$ ^doare<n><m><sg>/manière<n><f><sg>$ ^ober<vblex><inf>/faire<vblex><inf>$ ^.<sent>/.<sent>$^.<sent>/.<sent>$       0.0082725939    ||

#	 d) Crispiness threshold

class Counter(BCC.BiltransCounter):
    tokenizer = 'biltrans'
    line_ids = True
    count_ngrams = True
    max_ngrams = 3
    
def biltrans_count_patterns_ngrams(biltrans_ambig, biltrans_annotated, crisphold=3.0):
    # First read in the frequency defaults

    print('Reading...', file=sys.stderr)
    sys.stderr.flush()

    c = Counter()
    c.read_files(biltrans_ambig,  # File with ambiguous biltrans output
                biltrans_annotated)  # File with disambiguated biltrans output
    ngrams = c.ngrams

    print('Caching counts...', file=sys.stderr)
    for sl in ngrams:
        for ngram in ngrams[sl]:
            for tl in ngrams[sl][ngram]:
                print('%.10f\t%s\t%s\t%s' % (ngrams[sl][ngram][tl], ngram, sl, tl))

    print('\n', file=sys.stderr)

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: biltrans-count-patterns-ngrams.py <biltrans_ambig> <biltrans_annotated> [crisphold]', file=sys.stderr)
        exit(1)
    
    if len(sys.argv) == 4:
        print('crisp:', sys.argv[3], file=sys.stderr)
        biltrans_count_patterns_ngrams(sys.argv[1], sys.argv[2], sys.argv[3])
    else:
        biltrans_count_patterns_ngrams(sys.argv[1], sys.argv[2])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import random


def merge_ngrams_lambdas(ngf, ldf):
    # ngram file
    # lambda file
    # lexicon file

    ngrams = {}
    # lambdas = {}

    for line in open(ngf).readlines():
        # 59763 	poor<adj> in<pr> capital<adj>
        if len(line) < 2:
            continue

        row = line.strip().split('\t')
        if(len(row) < 2):
            row.append('')
        ngid = int(row[0].strip())
        ngrams[ngid] = row[1]

    with open(ldf) as d:
        for line in d:
            # 59176:0 1.00131
            if line.count('@@') > 0:
                continue

            row = line.strip().split('\t')

            l = float(row[2])
            ngid = int(row[1].split(':')[0])
            ngram = ngrams[ngid]

            trad = row[1].split(':')[1]
            token = row[0]
            print(token, '\t', l, '\t', trad, '\t', ngram)


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: merge-ngrams-lambdas.py <ngrams> <lambdas>', file=sys.stderr)
        exit(1)

    merge_ngrams_lambdas(sys.argv[1], sys.argv[2])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
from collections import defaultdict
import biltrans_count_common as BCC
import common

# Input:
#        a) Biltrans output
# 56011   ^un<det><ind><sp>/un<det><ind><GD><ND>$ ^digarez<n><m><sg>/excuse<n><f><sg>/occasion<n><f><sg>$ ^da<pr>/à<pr>$ ^distreiñ<vblex><inf>/revenir<vblex><inf>$ ^war<pr>/sur<pr>$ ^e<det><pos><m><sp>/son<det><pos><GD><ND>$ ^doare<n><m><sg>/manière<n><f><sg>$ ^ober<vblex><inf>/faire<vblex><inf>$ ^.<sent>/.<sent>$
#
#        b) Disambiguated biltrans output
# .[][56011 0].[] ^un<det><ind><sp>/un<det><ind><GD><ND>$ ^digarez<n><m><sg>/excuse<n><f><sg>$ ^da<pr>/à<pr>$ ^distreiñ<vblex><inf>/revenir<vblex><inf>$ ^war<pr>/sur<pr>$ ^e<det><pos><m><sp>/son<det><pos><GD><ND>$ ^doare<n><m><sg>/manière<n><f><sg>$ ^ober<vblex><inf>/faire<vblex><inf>$ ^.<sent>/.<sent>$^.<sent>/.<sent>$ 0.9917274061    |@|
# .[][56011 1].[] ^un<det><ind><sp>/un<det><ind><GD><ND>$ ^digarez<n><m><sg>/occasion<n><f><sg>$ ^da<pr>/à<pr>$ ^distreiñ<vblex><inf>/revenir<vblex><inf>$ ^war<pr>/sur<pr>$ ^e<det><pos><m><sp>/son<det><pos><GD><ND>$ ^doare<n><m><sg>/manière<n><f><sg>$ ^ober<vblex><inf>/faire<vblex><inf>$ ^.<sent>/.<sent>$^.<sent>/.<sent>$       0.0082725939    ||
#
#

class Counter(BCC.BiltransCounter):
    tokenizer = 'biltrans'
    line_ids = True
    # The sl-tl possible combinations
    sl_tl = defaultdict(lambda: defaultdict(lambda: 0.0))


    def process_lu(self, sl, tl, idx, cur_sl_row, frac_count=0):
        self.sl_tl[sl][tl] += frac_count

def biltrans_extract_frac_freq(biltrans_ambig, biltrans_annotated):

    c = Counter()
    c.read_files(biltrans_ambig,  # File with ambiguous biltrans output
                biltrans_annotated)  # File with disambiguated biltrans output

    for sl in c.sl_tl:
        newtl = sorted(c.sl_tl[sl], key=lambda x: c.sl_tl[sl][x])
        newtl.reverse()
        first = True
        for tl in newtl:
            if first:
                print('%.10f %s %s @' %
                    (c.sl_tl[sl][tl], common.wrap(sl), common.wrap(tl)))
                first = False
            else:
                print('%.10f %s %s' %
                    (c.sl_tl[sl][tl], common.wrap(sl), common.wrap(tl)))

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: biltrans-extract-frac-freq.py <biltrans_ambig> <biltrans_annotated>', file=sys.stderr)
        exit(1)
    
    biltrans_extract_frac_freq(sys.argv[1], sys.argv[2])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-


import re
import sys

re_start = re.compile('(^[^\^]*)')


def ambiguous(bt):
    # legislation<n><sg>/legislación<n><f><sg>/ordenamiento<n><m><sg>
    for token in bt:
        if len(token['tls']) > 1:
            return True

    return False


def wrap(x):
    return '^' + x + '$'


def parse_tags(ptr, line):
    tags = []
    tag = ''

    while True:
        c = line[ptr]

        if c == '$' or c == '/':
            return (ptr-1, tags)
        elif c == '>':
            tags.append(tag)
            tag = ''
        elif c != '<':
            tag += c

        ptr += 1


def parse_sl(ptr, line):
    out = ''
    if line[ptr] == '*':
        (ptr, out) = parse_unknown(ptr, line)
        return (ptr, (out, []))

    escaped = False
    while True:
        c = line[ptr]
        if c == '\\':
            escaped = True
        elif (c == '/' or c == '$') and not escaped:
            return (ptr, out)
        elif c == '<' and not escaped:
            (ptr, tags) = parse_tags(ptr+1, line)
            return (ptr, (out, tags))
        else:
            out += c
            escaped = False
        ptr += 1


def parse_unknown(ptr, line):
    out = ''
    escaped = False
    while True:
        c = line[ptr]
        if c == '\\':
            escaped = True
        elif (c == '$' or c == '/') and not escaped:
            return (ptr, out)
        else:
            out += c
            escaped = False
        ptr += 1


def parse_tls(ptr, line):
    tls = []
    tl = ''
    out = ''
    escaped = False
    if line[ptr] == '*':
        (ptr, out) = parse_unknown(ptr, line)
        return (ptr, [(out, [])])

    while True:
        if ptr == len(line):
            tls.append(tl)
            return (ptr, tls)
        c = line[ptr]
        if c == '\\':
            escaped = True
        elif c == '/' and tl != '' and not escaped:
            tls.append(tl)
            tl = ''
        elif c == '$' and not escaped:
            if tl != '':
                tls.append(tl)
            return (ptr, tls)
        elif c == '<' and not escaped:
            (ptr, tags) = parse_tags(ptr, line)
            tls.append((tl, tags))
            tl = ''
        elif c != '/' or escaped:
            tl += c
            escaped = False
        ptr += 1


def toBiltransToken(sl, tls):
    new_tls = []
    for tl in tls:
        new_tl = tl[0] + '<' + '><'.join(tl[1]) + '>'
        new_tls.append(new_tl)
    new_sl = sl[0] + '<' + '><'.join(sl[1]) + '>'

    return (new_sl, new_tls)


def parse_biltrans_token(ptr, line):
    (ptr, sl) = parse_sl(ptr, line)
    (ptr, tls) = parse_tls(ptr+1, line)
    (sl, tls) = toBiltransToken(sl, tls)

    token = {}
    token['sl'] = sl
    token['tls'] = tls

    return (ptr, token)


def parse_tagger_token(ptr, line):
    (ptr, sl) = parse_sl(ptr, line)
    sl = sl[0] + '<' + '><'.join(sl[1]) + '>'

    return (ptr, sl)


def tokenize_biltrans_line(line):
    return tokenise_biltrans_line(line)


def tokenise_biltrans_line(line):
    out = []
    escaped = False
    for ptr in range(0, len(line)):
        c = line[ptr]
        if c == '^' and not escaped:
            (ptr, token) = parse_biltrans_token(ptr+1, line)
            out.append(token)
        elif c == '\\':
            escaped = True
        elif escaped:
            escaped = False

    return out


def tokenize_tagger_line(line):
    return tokenise_tagger_line(line)


def tokenise_tagger_line(line):

    out = []
    escaped = False
    for ptr in range(0, len(line)):
        c = line[ptr]
        if c == '^' and not escaped:
            (ptr, token) = parse_tagger_token(ptr+1, line)
            out.append(token)
        elif c == '\\':
            escaped = True
        elif escaped:
            escaped = False

    return out


def tokenize_biltrans_line2(line):
    return tokenise_biltrans_line2(line)


def tokenise_biltrans_line2(line):
    line = clean_biltrans_line(line)[1:-1]
    row = []
    token = ''
    state = 0

    escaped = False

    for c in line:
        # in token
        if state == 0:
            if c == '$':
                row.append(token)
                token = ''
                state = 1
            elif c == '\\':
                continue
            else:
                token += c
        # between tokens
        elif state == 1:

            if c == '\\':
                escaped = True
            elif c == '^' and not escaped:
                state = 0
                escaped = False
            elif escaped:
                escaped = False

    return row


def clean_biltrans_line(line):
    line = re_start.sub('', line)
    return line



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common


def extract_alig_lrx(lex_freq):
    with open(lex_freq) as d:
        print('<rules>')

        for line in d:
            sys.stdout.flush()
            if line[-2] == '@':
                row = common.tokenize_tagger_line(line)

                fq = line.split(' ')[0]
                sl = row[0]
                tl = row[1]

                if line.count('>') < 2:
                    continue

                print(sl, tl, file=sys.stderr)
                sl_lem = sl.split('<')[0]
                tl_lem = tl.split('<')[0]
                sl_lem = sl_lem.replace(
                    '-', '\\-').replace('~', ' ').replace('&', '&amp;')
                tl_lem = tl_lem.replace(
                    '-', '\\-').replace('~', ' ').replace('&', '&amp;')

                sl_tag = sl.replace('><', '.').split('<')[1].strip('>')
                tl_tag = tl.replace('><', '.').split('<')[1].strip('>')

                cmb = ''
                cma = ''

                if sl_tag.split('.')[0] not in ['adj', 'vblex', 'n']:
                    cmb = '<!--'
                    cma = '-->'
                else:
                    cma = ''
                    cmb = ''

                rule = cmb + '<rule comment="' + fq + '">'
                # rule = rule + '<match lemma="' + sl_lem + '" tags="' + sl_tag + '"><select lemma="' + tl_lem + '" tags="' + tl_tag + '"/>'
                rule = rule + '<match lemma="' + sl_lem + '"><select lemma="' + tl_lem + '"/>'
                rule = rule + '</match>'
                rule = rule + '</rule>' + cma

                print(rule)

        print('</rules>')


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('Usage: extract-alig-lrx.py <lex_freq>', file=sys.stderr)
        exit(1)

    extract_alig_lrx(sys.argv[1])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common

#+nature<n>	service<n> nature<n>	carácter<n>	3
# +nature<n>	The<def><def> imperialist<adj> nature<n>	carácter<n>	1
# +nature<n>	the<def><def> secular<adj> nature<n> of<pr> State<n>	carácter<n>	1
# +nature<n>	its<det><pos> nature<n> prevent<vblex>	carácter<n>	1
# +nature<n>	nature<n> be<vbser> in<pr>	carácter<n>	1

def ngrams_to_rules(ngrams, crisphold):
    permitted_tags = ['n', 'vblex', 'adj', 'n.*', 'vblex.*', 'adj.*']

    print('<rules>')
    lineno = 1
    ruleno = 0
    for line in open(ngrams).readlines():
        #	print('\n')
        #	print(line)
        if len(line) < 2:
            continue

        line = line.strip()
        #line = line.strip()

        # + 0.571428571429 14 8 8 	troiñ<vblex>		tourner<vblex>	8
        row = line.split('\t')

        if len(row) == 3:
            row.insert(0, '')

    #	tipus = row[0].split(' ')[0]
        weight = row[0].split(' ')[1]
        sl = row[1].strip()[1:-1]
        tl = row[3][1:-1]
        tl_lema = tl.split('<')[0].lower()
        tl_tags = '<'.join(tl.split('<')[1:]).replace(
            '><', '.').replace('>', '')

        freq = row[4]
        pattern = common.tokenize_tagger_line(row[2])

        if row[2].count('<guio>') > 0 or row[2].count('<sent>') > 0 or row[2].count('<cm>') > 0:
            print('PUNCTUATION_IN_PATTERN', line, file=sys.stderr)
            continue

        inpattern = False
        for w in pattern:
            if w.count(sl) > 0:
                inpattern = True

        if inpattern == False:
            print('SL_NOT_IN_PATTERN', line, sl, tl, file=sys.stderr)
            continue

        if tl_tags.count('adj') > 0 and sl.count('adj') < 1:
            print("TAG_MISMATCH", line, file=sys.stderr)
            continue

        if tl_tags.count('vbmod') > 0 and sl.count('vbmod') < 1:
            print("TAG_MISMATCH", line, file=sys.stderr)
            continue

        if tl_tags.split('.')[0] not in permitted_tags:
            print("TAG_NOT_PERMITTED", tl_tags, '||', line, file=sys.stderr)
            continue

        if float(weight) <= float(crisphold):
            print("UNDER_THRESHOLD", weight, "<",
                  crisphold, "||",  line, file=sys.stderr)
            continue

        if any([x.startswith("*") for x in pattern]):
            print("UNKNOWN_WORD_IN_PATTERN", pattern, file=sys.stderr)
            continue

        sel = False
        ruleno = ruleno + 1
        lineno = lineno + 1

        print('  <rule c="' + str(ruleno) + ' ' + str(lineno) +
              ': ' + freq + '" weight="' + weight + '">')
        for word in pattern:
            sl_lema = word.split('<')[0].lower()
            if (sl_lema[0] == '*'):
                continue

            if word.count('><') > 0:
                sl_tags = '<'.join(word.split('<')[1:]).replace(
                    '><', '.').replace('>', '')
            else:
                sl_tags = '<'.join(word.split('<')[1:]).strip('<>')

            # ======================================================================= #

            sl_lema = sl_lema.replace('~', ' ')
            tl_lema = tl_lema.replace('~', ' ')
            sl_lema = sl_lema.replace('-', '\-')
            tl_lema = tl_lema.replace('-', '\-')
            sl_lema = sl_lema.replace('(', '\(')
            tl_lema = tl_lema.replace('(', '\(')
            sl_lema = sl_lema.replace(')', '\)')
            tl_lema = tl_lema.replace(')', '\)')

            if word.lower().count(sl) > 0:
                lineno = lineno + 1
                if sl_lema == '':
                    print('    <match tags="' + sl_tags + '"><select lemma="' +
                          tl_lema + '" tags="' + tl_tags + '"/></match>')
                else:
                    print('    <match lemma="' + sl_lema + '" tags="' + sl_tags +
                          '"><select lemma="' + tl_lema + '" tags="' + tl_tags + '"/></match>')

                sel = True
            else:
                lineno = lineno + 1
                if sl_lema == '':
                    print('    <match tags="' + sl_tags + '"/>')
                else:
                    print('    <match lemma="' + sl_lema +
                          '" tags="' + sl_tags + '"/>')

        if sel == False:

            print('  </rule> <!-- Warning: No select operation ', line, '-->')
        else:
            print('  </rule>')

        lineno = lineno + 1
    print('</rules>')


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: ngrams-to-rules.py <ngrams> <crisphold>', file=sys.stderr)
        exit(1)

    ngrams_to_rules(sys.argv[1], sys.argv[2])



import sys
import common


def wrap(x):
    return '^' + x + '$'


def lambdas_to_rules(freq_lexicon, rules):
    sl_tl_defaults = {}
    sl_tl = {}

    indexes = {}
    trad_counter = {}
    rindex = {}

    with open(freq_lexicon) as d:
        for line in d:
            if len(line) < 1:
                continue

            row = common.tokenise_tagger_line(line)
            sl = wrap(row[0].strip())
            tl = wrap(row[1].strip())
            if tl[1] == '*':
                tl = tl[:-3] + '$'

            if sl not in sl_tl:
                sl_tl[sl] = []

            if sl not in trad_counter:
                trad_counter[sl] = 0

            if line.count('@') > 0:
                sl_tl_defaults[sl] = tl

            sl_tl[sl].append(tl)
            indexes[(sl, tl)] = trad_counter[sl]
            rindex[(sl, trad_counter[sl])] = tl
            trad_counter[sl] = trad_counter[sl] + 1

    for pair in rindex:
        print(pair[0], pair[1], rindex[pair], file=sys.stderr)

    # ability<n> 	 0.25652 	 1 	 ability<n> to<pr>
    # ability<n> 	 1.54548 	 0 	 ability<n> to<pr> deliver<vblex><inf>
    # ability<n> 	 1.48162 	 0 	 our<det><pos> ability<n> to<pr>

    with open(rules) as d:
        for line in d:

            row = line.split(' \t ')
            slword = row[0].strip()
            l = float(row[1])
            tlid = int(row[2])
            if (slword, tlid) not in rindex:
                print('(', slword, ',', tlid, ') not in index', file=sys.stderr)
                continue

            tlword = rindex[(slword, tlid)]
            context = row[3].strip()
        #	#+ 0.571428571429 14 8 8 	troiñ<vblex>		tourner<vblex>	8
        #+nature<n>	service<n> nature<n>	carácter<n>	3

            print('+ ' + row[1] + '\t' + slword +
                  '\t' + context + '\t' + tlword + '\t1')

        #	print('  <rule weight="%.5f">' % (l))
        #	for c in context.split(' '):
        #		if c.count(slword) == 1:
        #			print(slword, tlword)
        #		else:
        #			print(c)
        #	print('  </rule>')


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: lambdas-to-rules.py <lex> <rules>', file=sys.stderr)
        exit(1)

    lambdas_to_rules(sys.argv[1], sys.argv[2])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common
import biltrans_count_common as BCC

# Input:

# 0.6000015452	k<post> bukatu<vblex> ari izan<vbper>	bukatu<vblex>	acabar<vblex>
# 0.3999984548	k<post> bukatu<vblex> ari izan<vbper>	bukatu<vblex>	terminar<vblex>
# 0.2435956440	a<det> eta<cnjcoo> bukatu<vblex>	bukatu<vblex>	acabar<vblex>
# 0.7564043560	a<det> eta<cnjcoo> bukatu<vblex>	bukatu<vblex>	terminar<vblex>
# 0.0003531084	eta<cnjcoo> *ed bukatu<vblex> izan<vbsint> n<rel>	bukatu<vblex>	acabar<vblex>
# 0.9996468916	eta<cnjcoo> *ed bukatu<vblex> izan<vbsint> n<rel>	bukatu<vblex>	terminar<vblex>
# 0.4520909033	*Jazten bukatu<vblex>	bukatu<vblex>	acabar<vblex>
# 0.5479090967	*Jazten bukatu<vblex>	bukatu<vblex>	terminar<vblex>

#	 d) Crispiness threshold

def ngram_pruning_frac(lex_freq, ngrams_file, crisphold=3.0):
    cur_line = 0
    only_max = True

    ngrams = {}

    # First read in the frequency defaults
    _, sl_tl_defaults, _ = BCC.read_frequencies(lex_freq)

    max_crispiness = 0.0
    print('Reading...', file=sys.stderr)

    # Load counts from cached file

    ngramsf = open(ngrams_file)
    for line in ngramsf.readlines():
        if len(line) < 1:
            continue

        row = line.split('\t')

        freq = float(row[0])
        ngram = row[1]
        sl = row[2]
        tl = row[3].strip()

        if sl not in ngrams:
            ngrams[sl] = {}

        if ngram not in ngrams[sl]:
            ngrams[sl][ngram] = {}

        if tl not in ngrams[sl][ngram]:
            ngrams[sl][ngram][tl] = 0.0

        ngrams[sl][ngram][tl] = freq


    for sl in ngrams:
        if sl == '':
            continue

        for ngram in ngrams[sl]:
            if ngram == '':
                continue

            total = 0.0
            max_freq = -1.0
            max_tl = ''
            for tl in ngrams[sl][ngram]:

                if ngrams[sl][ngram][tl] > max_freq:
                    max_freq = ngrams[sl][ngram][tl]
                    max_tl = tl

                total = total + ngrams[sl][ngram][tl]

            default = sl_tl_defaults[sl]

            if max_tl not in ngrams[sl][ngram] and default not in ngrams[sl][ngram]:
                print('Some shit went down..', file=sys.stderr)
                print('= %s\t%s\t%s' % (sl, ngram, max_tl), file=sys.stderr)
                continue

            if max_freq == 0.0:
                continue

            if only_max == True:
                crispiness = 0.0
                alt_crisp = float(ngrams[sl][ngram][max_tl]) / float(total)
                def_crisp = 1.0
                if default in ngrams[sl][ngram]:
                    def_crisp = float(ngrams[sl][ngram][default] / float(total))

                if def_crisp == 0.0:
                    print('!!! Something wanky happened. :(', file=sys.stderr)
                    print('%.10f %.10f %.10f\t%s\t%s\t%s\t%.10f' % (
                        total, max_freq, ngrams[sl][ngram][max_tl], sl, ngram, max_tl, ngrams[sl][ngram][max_tl]), file=sys.stderr)
                    print('\tskipping...', file=sys.stderr)
                    continue

                weight = float(ngrams[sl][ngram][max_tl]) / float(total)
                crispiness = alt_crisp/def_crisp

                print('- %.10f %.10f %.10f %.10f %.10f\t%s\t%s\t%s\t%.10f' % (crispiness, weight, total,
                                                                            max_freq, ngrams[sl][ngram][max_tl], sl, ngram, max_tl, ngrams[sl][ngram][max_tl]))
                # print('- %.10f \t%s\t%s\t%s\t%.10f' % (crispiness, sl, ngram, max_tl, ngrams[sl][ngram][max_tl]));

                if crispiness > max_crispiness:
                    max_crispiness = crispiness


            #   crispiness   weight      total default     max_freq     tl_freq            sl
            # + 2.61845457309 0.7236389238 1.0 0.2763610762 0.7236389238 0.7236389238         aozer<n>        aozer<n> an<det> levr<n>        organisateur<n> 0.7236389238
            # - 14736.0468727 0.9999321438 1.0 0.9999321438 0.9999321438      treuzkas<n>     treuzkas<n> teknologel<adj>     transfert<n>    0.9999321438
            else:

                for tl in ngrams[sl][ngram]:

                    crispiness = 0.0
                    default = sl_tl_defaults[sl]
                    alt_crisp = float(ngrams[sl][ngram][tl]) / float(total)
                    def_crisp = 1.0
                    if default in ngrams[sl][ngram]:
                        def_crisp = float(
                            ngrams[sl][ngram][default] / float(total))

                    weight = float(ngrams[sl][ngram][tl]) / float(total)
                    crispiness = alt_crisp/def_crisp

                    # print '%%%' , crispiness , alt_crisp , def_crisp , tl , default , ngrams[sl][ngram] ;

                    print('- %.10f %.10f %.10f %.10f %.10f %.10f\t%s\t%s\t%s\t%.10f' % (crispiness, weight, total,
                                                                                        ngrams[sl][ngram][default], max_freq, ngrams[sl][ngram][tl], sl, ngram, tl, ngrams[sl][ngram][tl]))
                    # + 1013.01568891 0.9989973752 2.0 1.9979947504 1.9979947504 	galloud<n>	ha<cnjcoo> an<det> galloud<n>	puissance<n>	1.9979947504

                if crispiness > max_crispiness:
                    max_crispiness = crispiness


    print('max_crispiness: %.10f' % (max_crispiness), file=sys.stderr)

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: ngram-pruning-frac.py <lex_freq> <ngrams> [crisphold]', file=sys.stderr)
        exit(1)
    
    if len(sys.argv) == 4:
        print('crisp:', sys.argv[3], file=sys.stderr)
        ngram_pruning_frac(sys.argv[1], sys.argv[2], sys.argv[3])
    else:
        ngram_pruning_frac(sys.argv[1], sys.argv[2])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common

# Read the corpus, make a note of all ambiguous words, their frequency and their possible translations

# sl_tl[sl_word][tl_word] = tl_freq

# Then we want to make a list of python $SCRIPTS/ngram-count-patterns-maxent.py $TRAIN/$CORPUS.lex.$SL-$TL $TRAIN/$CORPUS.candidates.$SL-$TL 2>ngrams > events n-grams around the source words, with which target word they want, and the freq.

# ngrams[ngram][tl_word] = freq

# 5 	Please<vblex><inf> rise<n> ,<cm> then<adv> ,<cm> for<pr> this<det><dem> minute<n> 's<gen> silence<n> .<sent>
# 5 	Please<vblex><inf>/Complacer<vblex><inf> rise<n><sg>/aumento<n><m><sg> ,<cm>/,<cm> then<adv>/entonces<adv> ,<cm>/,<cm> for<pr>/para<pr>/durante<pr> this<det><dem><sg>/este<det><dem><GD><sg> minute<n><sg>/minuto<n><m><sg> '<apos>/'<apos> *s/*s silence<n><sg>/silencio<n><m><sg> .<sent>/.<sent>
# 5 	Invitar<vblex> a<pr> todo<prn><tn> a<pr> que<cnjsub> prpers<prn><pro> poner<vblex> de<pr> pie<n> para<pr> guardar<vblex><inf> uno<det><ind> minuto<n> de<pr> silencio<n> .<sent>
# 5 	0-0 4-2 5-3 8-1 9-5 10-6 12-7 13-8 14-9 15-10
# -------------------------------------------------------------------------------

THRESHOLD = 0


def wrap(x):
    return '^' + x + '$'


def ngram_count_patterns(freq_lexicon, candidates):

    MAX_NGRAMS = 3
    cur_line = 0

    sl_tl_defaults = {}
    sl_tl = {}
    ngrams = {}

    meevents = {}  # events[slword][counter] = [feat, feat, feat]
    meoutcomes = {}  # meoutcomes[slword][counter] = tlword
    event_counter = 0

    features = {}  # features[(slword, ['a', 'list'], tlword)] = 3
    feature_counter = 0

    indexes = {}
    trad_counter = {}
    for line in open(freq_lexicon, 'r').readlines():
        if len(line) < 1:
            continue

        w = int(line.split(' ')[0])
        if w < THRESHOLD:
            continue

        row = common.tokenise_tagger_line(line)
        sl = wrap(row[0]).lower()
        tl = wrap(row[1]).lower()
        if tl[1] == '*':
            tl = tl[:-3] + '$'

        if sl not in sl_tl:
            sl_tl[sl] = []
        
        if sl not in trad_counter:
            trad_counter[sl] = 0

        if line.count('@') > 0:
            sl_tl_defaults[sl] = tl
        sl_tl[sl].append(tl)
        indexes[(sl, tl)] = trad_counter[sl]
        trad_counter[sl] = trad_counter[sl] + 1

    cur_sl_row = []
    cur_tl_row = []
    cur_bt_row = []
    cur_al_row = []

    for line in open(candidates, 'r').readlines():
        line = line.strip()
        if line[0] == '-':
            # Read the corpus, make a note of all ambiguous words, their frequency and their possible translations
            #
            # sl_tl[sl_word][tl_word] = tl_freq
            i = 0
            for slword in cur_sl_row:
                if len(cur_bt_row[i]['tls']) > 1:
                    for al in cur_al_row:
                        al_sl = int(al.split('-')[1])
                        al_tl = int(al.split('-')[0])
                        if al_sl != i:
                            continue
                        

                        tlword = wrap(cur_tl_row[al_tl].lower())
                        slword = wrap(slword.lower())

                        if tlword[1] == '*' or slword[1] == '*':
                            continue

                        if slword not in sl_tl_defaults:
                            #						print >>sys.stderr, 'WARNING: "' + slword + '" not in sl_tl_defaults, skipping'
                            continue
                        
                        if (slword, tlword) not in indexes:
                            #						print >>sys.stderr, 'WARNING: pair (%s, %s) not found in index' % (slword, tlword)
                            continue
                        
    #					if tlword !=  sl_tl_defaults[slword]:
    #						print >>sys.stderr, '+' , slword , sl_tl_defaults[slword] , tlword
    #					else:
    #						print >>sys.stderr, '-' , slword , sl_tl_defaults[slword] , tlword
    #					print >>sys.stderr, cur_sl_row
                        for j in range(1, MAX_NGRAMS):
                            #						print >>sys.stderr, cur_sl_row[i] , cur_sl_row[i-j:i+1]
                            #						print >>sys.stderr, cur_sl_row[i] , cur_sl_row[i:i+j+1]
                            #						print >>sys.stderr, cur_sl_row[i] , cur_sl_row[i-j:i+j+1]

                            pregram = ' '.join(map(wrap, cur_sl_row[i-j:i+1]))
                            postgram = ' '.join(map(wrap, cur_sl_row[i:i+j+1]))
                            roundgram = ' '.join(
                                map(wrap, cur_sl_row[i-j:i+j+1]))

                            if slword not in ngrams:
                                ngrams[slword] = {}
                            
                            if pregram not in ngrams[slword]:
                                ngrams[slword][pregram] = {}
                            
                            if postgram not in ngrams[slword]:
                                ngrams[slword][postgram] = {}
                            
                            if roundgram not in ngrams[slword]:
                                ngrams[slword][roundgram] = {}
                            
                            if tlword not in ngrams[slword][pregram]:
                                ngrams[slword][pregram][tlword] = 0
                            
                            if tlword not in ngrams[slword][postgram]:
                                ngrams[slword][postgram][tlword] = 0
                            
                            if tlword not in ngrams[slword][roundgram]:
                                ngrams[slword][roundgram][tlword] = 0
                            

                            ngrams[slword][pregram][tlword] = ngrams[slword][pregram][tlword] + 1
                            ngrams[slword][postgram][tlword] = ngrams[slword][postgram][tlword] + 1
                            ngrams[slword][roundgram][tlword] = ngrams[slword][roundgram][tlword] + 1
                        
                        # print ',' , len(ngrams[slword])
                        if slword not in meevents:
                            meevents[slword] = {}
                        
                        if slword not in meoutcomes:
                            meoutcomes[slword] = {}
                        
                        if event_counter not in meevents:
                            meevents[slword][event_counter] = []
                        
                        if event_counter not in meoutcomes[slword]:
                            meoutcomes[slword][event_counter] = ''
                        
                        for ni in ngrams[slword]:
                            if ni not in features:
                                feature_counter = feature_counter + 1
                                features[ni] = feature_counter
                            
                            meevents[slword][event_counter].append(
                                features[ni])
                            # meevents[slword][event_counter].append(feat)
                            meoutcomes[slword][event_counter] = tlword

                        
                        del ngrams
                        ngrams = {}
                        if len(sl_tl[slword]) < 2:
                            continue
                        
                        for event in meevents[slword]:
                            outline = str(
                                indexes[(slword, meoutcomes[slword][event])]) + ' # '
                            for j in range(0,  len(sl_tl[slword])):
                                for feature in meevents[slword][event]:
                                    outline = outline + \
                                        str(feature) + ':' + str(j) + ' '
                                
                                outline = outline + ' # '
                            
                            print(slword, '\t', len(
                                sl_tl[slword]), '\t', outline)
                        
                        del meevents
                        del meoutcomes
                        meevents = {}
                        meoutcomes = {}

    #					for f in features:
    #						print >>sys.stderr, features[f] , f

                    

    #				for j in range(0, MAX_NGRAMS):
    #					print cur_sl_row[i-j:i+1]
    #					print cur_sl_row[i:i+j]
                    # print ngrams[slword]
                
                i = i + 1

            

            cur_line = 0
            event_counter = event_counter + 1
            # print line
            continue
        

        line = line.split('\t')[1]
        line = line.strip()

        if cur_line == 0:
            cur_sl_row = common.tokenise_tagger_line(line)
        elif cur_line == 1:
            cur_bt_row = common.tokenise_biltrans_line(line)
        elif cur_line == 2:
            cur_tl_row = common.tokenise_tagger_line(line)
        elif cur_line == 3:
            cur_al_row = line.split(' ')
        

        cur_line = cur_line + 1
    

    for feature in features:
        print(features[feature], '\t', feature, file=sys.stderr)
    

    # exit(1)
    return

    # for slword in meevents:
    #     if len(sl_tl[slword]) < 2:
    #         continue
        
    #     for event in meevents[slword]:
    #         outline = str(indexes[(slword, meoutcomes[slword][event])]) + ' # '
    #         for j in range(0,  len(sl_tl[slword])):
    #             for feature in meevents[slword][event]:
    #                 outline = outline + str(feature) + ':' + str(j) + ' '
                
    #             outline = outline + ' # '
            
    #         print(slword, '\t', len(sl_tl[slword]), '\t', outline)
        
    


if __name__ == '__main__':
    if len(sys.argv) not in [3, 4]:
        print(
            'Usage: count-patterns.py <lex> <candidates> [threshold]', file=sys.stderr)
        exit(1)
    

    if len(sys.argv) == 4:
        THRESHOLD = int(sys.argv[3])

    ngram_count_patterns(sys.argv[1], sys.argv[2])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common

# +nature<n>	service<n> nature<n>	carácter<n>	3
# +nature<n>	The<def><def> imperialist<adj> nature<n>	carácter<n>	1
# +nature<n>	the<def><def> secular<adj> nature<n> of<pr> State<n>	carácter<n>	1
# +nature<n>	its<det><pos> nature<n> prevent<vblex>	carácter<n>	1
# +nature<n>	nature<n> be<vbser> in<pr>	carácter<n>	1

def ngrams_to_rules(ngrams):
    # FREQMIN = 8.0

    MINMATCH = 2

    permitted_tags = ['n', 'vblex', 'adj']

    print('<rules>')
    lineno = 1
    ruleno = 0

    with open(ngrams) as infile:
        for line in infile:
            #	print '\n'
            #	print line
            if len(line) < 2:
                continue

            line = line.strip()
            # line = line.decode('utf-8').strip()
            print(line, file=sys.stderr)
            # + 0.571428571429 14 8 8 	troiñ<vblex>		tourner<vblex>	8
            row = line.split('\t')

            tipus = row[0].split(' ')[0]
            weight = row[0].replace('  ', ' ').split(' ')[1]
            sl = row[1].strip()[1:-1]
            tl = row[3][1:-1]
            tl_lema = tl.split('<')[0].lower()
            tl_tags = ''.join(tl.split('<')[1:]).replace(
                '>', '.').rstrip('.')
            freq = 1
        #	freq = float(row[4])

            pattern = common.tokenize_tagger_line(row[2])

            if row[2].count('<guio>') > 0 or row[2].count('<sent>') > 0 or row[2].count('<cm>') > 0:
                print('PUNCTUATION_IN_PATTERN', line, file=sys.stderr)
                continue

            if tipus == '-' or tipus == '~':
                print('DEFAULT_READING', line, file=sys.stderr)
                continue

            # Hacks
        #	if len(pattern) == 0:
        #		print('ZERO_PATTERN' , line, file=sys.stderr);
        #		continue

            if len(pattern) < MINMATCH and len(pattern) > 0:
                print('BELOW_MINMATCH', line, file=sys.stderr)
                continue

            inpattern = False
            for w in pattern:
                if w.lower().count(sl) > 0:
                    inpattern = True

            if len(pattern) > 0 and not inpattern:
                print('SL_NOT_IN_PATTERN', line, file=sys.stderr)
                continue

            if tl_tags.count('adj') > 0 and sl.count('adj') < 1:
                print("TAG_MISMATCH", line, file=sys.stderr)
                continue

            if tl_tags.count('vbmod') > 0 and sl.count('vbmod') < 1:
                print("TAG_MISMATCH", line, file=sys.stderr)
                continue

            if tl_tags.split('.')[0] not in permitted_tags:
                print("TAG_NOT_PERMITTED", tl_tags,
                      '||', line, file=sys.stderr)
                continue

            sel = False
            ruleno = ruleno + 1
            lineno = lineno + 1

            commentb = ''
            commente = ''
        #	if freq < FREQMIN:
        #		commentb = '<!--'
        #		commente = '-->'

            print(commentb + '  <rule c="' + str(ruleno) + ' ' +
                  str(lineno) + ': ' + str(freq) + '" weight="' + weight + '">')
            for word in pattern:
                sl_lema = word.split('<')[0].lower()
                if word.count('><') > 0:
                    sl_tags = '<'.join(word.split('<')[1:]).replace(
                        '><', '.').replace('>', '')
                else:
                    sl_tags = '<'.join(word.split('<')[1:]).strip('<>')

                # ======================================================================= #

                sl_lema = sl_lema.replace('~', ' ')
                tl_lema = tl_lema.replace('~', ' ')
        #		sl_lema = sl_lema.replace('-', '\-')
        #		tl_lema = tl_lema.replace('-', '\-')
        #		sl_lema = sl_lema.replace('(', '\(')
        #		tl_lema = tl_lema.replace('(', '\(')
        #		sl_lema = sl_lema.replace(')', '\)')
        #		tl_lema = tl_lema.replace(')', '\)')
        #
                if word.lower().count(sl) > 0:
                    lineno = lineno + 1
                    if sl_lema == '':
                        print('    <match tags="' + sl_tags + '"><select lemma="' +
                              tl_lema + '" tags="' + tl_tags + '"/></match>')
                    else:
                        print('    <match lemma="' + sl_lema + '" tags="' + sl_tags +
                              '"><select lemma="' + tl_lema + '" tags="' + tl_tags + '"/></match>')

                    sel = True
                else:
                    lineno = lineno + 1
                    if sl_lema == '':
                        print('    <match tags="' + sl_tags + '"/>')
                    else:
                        print('    <match lemma="' + sl_lema +
                              '" tags="' + sl_tags + '"/>')

            if sel == False and len(pattern) == 0:
                sl_lema = sl.split('<')[0]
                if sl.count('><') > 0:
                    sl_tags = '<'.join(sl.split('<')[1:]).replace(
                        '><', '.').replace('>', '')
                else:
                    sl_tags = '<'.join(sl.split('<')[1:]).strip('<>')

                if sl_lema == '':
                    print('    <match tags="' + sl_tags + '"><select lemma="' +
                          tl_lema + '" tags="' + tl_tags + '"/></match>')
                else:
                    print('    <match lemma="' + sl_lema + '" tags="' + sl_tags +
                          '"><select lemma="' + tl_lema + '" tags="' + tl_tags + '"/></match>')
                print('  </rule>' + commente)
            elif sel == False:
                print('  </rule>'+commente +
                      '<!-- Warning: No select operation ', line, '-->')
            else:
                print('  </rule>' + commente)

            lineno = lineno + 1

    print('</rules>')


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('Usage: ngrams-to-rules-me.py <ngrams>', file=sys.stderr)
        exit(1)

    ngrams_to_rules(sys.argv[1])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common

# Read the corpus, make a note of all ambiguous words, their frequency and their possible translations

# sl_tl[sl_word][tl_word] = tl_freq

# Then we want to make a list of n-grams around the source words, with which target word they want, and the freq.

# ngrams[ngram][tl_word] = freq

# 5 	Please<vblex><inf> rise<n> ,<cm> then<adv> ,<cm> for<pr> this<det><dem> minute<n> 's<gen> silence<n> .<sent>
# 5 	Please<vblex><inf>/Complacer<vblex><inf> rise<n><sg>/aumento<n><m><sg> ,<cm>/,<cm> then<adv>/entonces<adv> ,<cm>/,<cm> for<pr>/para<pr>/durante<pr> this<det><dem><sg>/este<det><dem><GD><sg> minute<n><sg>/minuto<n><m><sg> '<apos>/'<apos> *s/*s silence<n><sg>/silencio<n><m><sg> .<sent>/.<sent>
# 5 	Invitar<vblex> a<pr> todo<prn><tn> a<pr> que<cnjsub> prpers<prn><pro> poner<vblex> de<pr> pie<n> para<pr> guardar<vblex><inf> uno<det><ind> minuto<n> de<pr> silencio<n> .<sent>
# 5 	0-0 4-2 5-3 8-1 9-5 10-6 12-7 13-8 14-9 15-10
# -------------------------------------------------------------------------------

def ngram_count_patterns(freq_lexicon, candidates, crisphold, max_rules):
    MAX_NGRAMS = 2
    cur_line = 0

    sl_tl_defaults = {}
    sl_tl = {}
    ngrams = {}

    lineno = 0
    for line in open(freq_lexicon).readlines():
        lineno += 1
        if lineno % 10000 == 0:
            print(lineno, file=sys.stderr)
        if len(line) < 1:
            continue

        row = common.tokenise_tagger_line(line)
        sl = common.wrap(row[0])
        tl = common.wrap(row[1])
        if tl[1] == '*':
            tl = tl[:-3] + '$'
        if line.count('@') > 0:
            sl_tl_defaults[sl] = tl
        else:
            sl_tl[sl] = tl

    cur_sl_row = []
    cur_tl_row = []
    cur_bt_row = []
    cur_al_row = []
    lineno = 0
    for line in open(candidates).readlines():
        lineno += 1
        line = line.strip()
        if lineno % 500 == 0:
            print(lineno, file=sys.stderr)
        if line[0] == '-':
            #		print len(cur_sl_row), len(cur_tl_row), len(cur_bt_row), len(cur_al_row)
            #		print cur_sl_row
            #		print cur_bt_row
            #		print cur_tl_row
            #		print cur_al_row
            #
            # Read the corpus, make a note of all ambiguous words, their frequency and their possible translations
            #
            # sl_tl[sl_word][tl_word] = tl_freq
            i = 0
            for slword in cur_sl_row:
                if len(cur_bt_row[i]['tls']) > 1:
                    for al in cur_al_row:
                        if al == '':
                            continue
                        al_sl = int(al.split('-')[1])
                        al_tl = int(al.split('-')[0])
                        if al_sl != i:
                            continue

                        tlword = common.wrap(cur_tl_row[al_tl])
                        slword = common.wrap(slword)

                        if slword not in sl_tl_defaults:
                            print('!', file=sys.stderr)
                            continue

                        for j in range(1, MAX_NGRAMS):
                            pregram = ' '.join(map(common.wrap, cur_sl_row[i-j:i+1]))
                            postgram = ' '.join(map(common.wrap, cur_sl_row[i:i+j+1]))
                            roundgram = ' '.join(
                                map(common.wrap, cur_sl_row[i-j:i+j+1]))

                            if slword not in ngrams:
                                ngrams[slword] = {}

                            if pregram not in ngrams[slword]:
                                ngrams[slword][pregram] = {}

                            if postgram not in ngrams[slword]:
                                ngrams[slword][postgram] = {}

                            if roundgram not in ngrams[slword]:
                                ngrams[slword][roundgram] = {}

                            if tlword not in ngrams[slword][pregram]:
                                ngrams[slword][pregram][tlword] = 0

                            if tlword not in ngrams[slword][postgram]:
                                ngrams[slword][postgram][tlword] = 0

                            if tlword not in ngrams[slword][roundgram]:
                                ngrams[slword][roundgram][tlword] = 0

                            ngrams[slword][pregram][tlword] = ngrams[slword][pregram][tlword] + 1
                            ngrams[slword][postgram][tlword] = ngrams[slword][postgram][tlword] + 1
                            ngrams[slword][roundgram][tlword] = ngrams[slword][roundgram][tlword] + 1

                i = i + 1

            cur_line = 0
            # print line
            continue

        line = line.split('\t')[1]

        if cur_line == 0:
            cur_sl_row = common.tokenise_tagger_line(line)
        elif cur_line == 1:
            cur_bt_row = common.tokenise_biltrans_line(line)
        elif cur_line == 2:
            cur_tl_row = common.tokenise_tagger_line(line)
        elif cur_line == 3:
            cur_al_row = line.split(' ')

        cur_line = cur_line + 1

    for sl in ngrams:
        for ngram in ngrams[sl]:
            total = 0
            max_freq = -1
            current_tl = ''
            newtl = sorted(ngrams[sl][ngram], key=lambda x: ngrams[sl][ngram][x])
            newtl.reverse()
            newtl = newtl[:max_rules]
            for tl in newtl:
                if ngrams[sl][ngram][tl] > max_freq:
                    max_freq = ngrams[sl][ngram][tl]
                    current_tl = tl

                total = total + ngrams[sl][ngram][tl]

            # > If for each of the rules we include
            # > the amount of time the translation is seen with that pattern over the
            # > total, we get a number we can try as a threshold. e.g. > 0.6 >0.7 >0.8
            # > etc.  (>0.6 would be the same as 2/3 of the time the alternative
            # > translation is seen with that ngram, and 1/3 of the time the default
            # > translation is). I think this would be easier to explain than the magic
            # > number I came up with.
            #
            # I see this as a way to define how "crispy" the decisions are. I think it
            # would be better to express this as a ratio: the ratio of the times the
            # alternative translation is seen to the number of times the defaullt
            # translation is seen with that n-gram.
            #
            # It would be "2" in this case: the alternative is seen twice as often as
            # the default.

            for tl in newtl:
                crispiness = 0.0
                default = sl_tl_defaults[sl]
                alt_crisp = float(ngrams[sl][ngram][tl]) / float(total)
                def_crisp = 1.0
                if default in ngrams[sl][ngram]:
                    def_crisp = float(
                        ngrams[sl][ngram][default] / float(total))

                weight = float(ngrams[sl][ngram][tl]) / float(total)
                crispiness = alt_crisp/def_crisp

                # print '%%%' , crispiness , alt_crisp , def_crisp , tl , default , ngrams[sl][ngram]

                if crispiness < crisphold:
                    print('-', crispiness, weight, total, max_freq,
                          ngrams[sl][ngram][tl], '\t' + sl + '\t' + ngram + '\t' + tl + '\t' + str(ngrams[sl][ngram][tl]))
                else:

                    print('+', crispiness, weight, total, max_freq,
                          ngrams[sl][ngram][tl], '\t' + sl + '\t' + ngram + '\t' + tl + '\t' + str(ngrams[sl][ngram][current_tl]))


if __name__ == '__main__':
    if len(sys.argv) < 5:
        print(
            'Usage: count-patterns.py <lex> <candidates> <crispiness threshold> <max_rules>', file=sys.stderr)
        exit(1)

    ngram_count_patterns(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])



#!/usr/local/bin/python3.13
# coding=utf-8
# -*- encoding: utf-8 -*-

import sys
import common
import traceback

# Read the corpus, make a note of all ambiguous words, their frequency and their possible translations

# sl_tl[sl_word][tl_word] = tl_freq

# Then we want to make a list of n-grams around the source words, with which target word they want, and the freq.

# ngrams[ngram][tl_word] = freq

# 5 	Please<vblex><inf> rise<n> ,<cm> then<adv> ,<cm> for<pr> this<det><dem> minute<n> 's<gen> silence<n> .<sent>
# 5 	Please<vblex><inf>/Complacer<vblex><inf> rise<n><sg>/aumento<n><m><sg> ,<cm>/,<cm> then<adv>/entonces<adv> ,<cm>/,<cm> for<pr>/para<pr>/durante<pr> this<det><dem><sg>/este<det><dem><GD><sg> minute<n><sg>/minuto<n><m><sg> '<apos>/'<apos> *s/*s silence<n><sg>/silencio<n><m><sg> .<sent>/.<sent>
# 5 	Invitar<vblex> a<pr> todo<prn><tn> a<pr> que<cnjsub> prpers<prn><pro> poner<vblex> de<pr> pie<n> para<pr> guardar<vblex><inf> uno<det><ind> minuto<n> de<pr> silencio<n> .<sent>
# 5 	0-0 4-2 5-3 8-1 9-5 10-6 12-7 13-8 14-9 15-10
# -------------------------------------------------------------------------------

def extract_freq_lexicon(canditates):

    cur_line = 0
    lineno = 0
    sl_tl = {}

    cur_sl_row = []
    cur_tl_row = []
    cur_bt_row = []
    cur_al_row = []

    with open(canditates) as infile:
        for line in infile:
            line = line.strip()
            lineno += 1
            if lineno % 5000 == 0:
                sys.stderr.write('.')
                if lineno % 100000 == 0:
                    sys.stderr.write(str(lineno)+'\n')

                sys.stderr.flush()

            try:
                if line[0] == '-':
                    # Read the corpus, make a note of all ambiguous words, their frequency and their possible translations
                    #
                    # sl_tl[sl_word][tl_word] = tl_freq
                    i = 0
                    for slword in cur_sl_row:
                        if len(cur_bt_row[i]['tls']) > 1:
                            for al in cur_al_row:
                                if al == '':
                                    continue
                                al_sl = int(al.split('-')[1])
                                al_tl = int(al.split('-')[0])
                                if al_sl != i:
                                    continue

                                if al_tl < len(cur_tl_row):
                                    tlword = cur_tl_row[al_tl]
                                else:
                                    tlword = cur_tl_row[-1]
                                    traceback.print_stack()
                                    print("alignment out",
                                          "of",
                                          "range", al_tl,
                                          "not in",
                                          "len(",
                                          cur_tl_row,
                                          ")",
                                          file=sys.stderr)
                                    exit(1)
                                slword = slword
                                if slword not in sl_tl:
                                    sl_tl[slword] = {}

                                if tlword not in sl_tl[slword]:
                                    sl_tl[slword][tlword] = 0

                                sl_tl[slword][tlword] = sl_tl[slword][tlword] + 1
                                # print '+' , slword , tlword , sl_tl[slword][tlword], lineno

                        i = i + 1

                    cur_line = 0
                    continue

                line = line.split('\t')[1]

                if cur_line == 0:
                    cur_sl_row = common.tokenise_tagger_line(line)
                elif cur_line == 1:
                    cur_bt_row = common.tokenise_biltrans_line(line)
                elif cur_line == 2:
                    cur_tl_row = common.tokenise_tagger_line(line)
                elif cur_line == 3:
                    cur_al_row = line.split(' ')

                cur_line = cur_line + 1
            except Exception:
                # print("Error in line", lineno, ":", e, file=sys.stderr)
                traceback.print_exc()
                exit(1)

    for sl in sl_tl:

        newtl = sorted(sl_tl[sl], key=lambda x: sl_tl[sl][x])
        newtl.reverse()
        first = True
        for tl in newtl:
            if tl[0] == '*':
                print('Error: tl word unknown', tl,  file=sys.stderr)
                continue

            first_tag_sl = sl.split('<')[1].split('>')[0].strip()
            first_tag_tl = tl.split('<')[1].split('>')[0].strip()
            if first_tag_sl != first_tag_tl:
                print('Error:', first_tag_sl, '!=',
                      first_tag_tl, file=sys.stderr)
                continue

            if first:
                print(sl_tl[sl][tl], common.wrap(sl), common.wrap(tl), '@')
                first = False
            else:
                print(sl_tl[sl][tl], common.wrap(sl), common.wrap(tl))


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('Usage: extract-freq-lexicon.py <candidate sent>', file=sys.stderr)
        exit(1)

    extract_freq_lexicon(sys.argv[1])
